-- TapLockTurbo
-- By Pizzasgood (Rin Ganborro)
-- This software is released under the WTFPL Version 2 (http://sam.zoy.org/wtfpl/COPYING)
-- (In short:  Do whatever you want with it.)

-- To use this, bind +taplockturbo to a key.
-- Press and hold for normal turbo functionality.
-- Tap twice quickly, and turbo will be locked.
-- To unlock, press and release the key once more.
-- This is automatically unlocked when you jump or undock.
-- You can adjust the maximum delay between taps:
--     /taplockturbo 250
-- The above would change the delay to 250 milliseconds.
-- Setting delay to 0 will disable locking.

TapLockTurbo = {}
TapLockTurbo.version = 1.0
TapLockTurbo.delay = gkini.ReadInt('TapLockTurbo', 'delay', 500)
TapLockTurbo.locked = false
TapLockTurbo.pressed = false
TapLockTurbo.tap_flag = false
TapLockTurbo.tap_timer = Timer()

local red = '\127FF0000'
local green = '\12700FF00'
local reset = '\127o'
local command_color = '\127FFFFFF'
local info_color = '\127D0D0D0'

--engage turbo
function TapLockTurbo:engage_turbo()
	gkinterface.GKProcessCommand('+turbo 1')
end


--disengage turbo
function TapLockTurbo:disengage_turbo()
	gkinterface.GKProcessCommand('+turbo 0')
end


--set maximum delay between taps, in ms
function TapLockTurbo:set_delay(delay)
	self.delay = delay
	gkini.WriteInt('TapLockTurbo', 'delay', self.delay)
	print(info_color..'Tap delay set to '..reset..command_color..delay..reset..info_color..' ms'..reset)
end


--function that runs when button pressed
function TapLockTurbo:press()
	if not self.pressed then
		if self.locked then
			--turbo was locked, so unlock now (but don't disengage yet)
			self.locked = false
			HUD:PrintSecondaryMsg('Turbo '..red..'unlocked'..reset)
		else
			--turbo was not locked, decide what to do
			self.tap_timer:Kill() -- no matter what, we'll want to kill the timer if it's running
			self:engage_turbo() -- no matter what, we'll want to engage turbo if it isn't already
			if self.tap_flag then
				--this is the second tap within self.delay ms, so set lock flag
				self.tap_flag = false
				self.locked = true
				HUD:PrintSecondaryMsg('Turbo '..green..'locked'..reset)
			else
				--this is the first tap within self.delay ms, so set tap flag and a timer to reset the flag
				if self.delay > 0 then
					self.tap_flag = true
					self.tap_timer:SetTimeout(self.delay,function()
						TapLockTurbo.tap_flag = false
					end);
				end
			end
		end
		self.pressed = true
	end
end


--function that runs when button released
function TapLockTurbo:release()
	--if turbo is not locked, disengage
	if not self.locked then
		self:disengage_turbo()
	end
	self.pressed = false
end


--process events
function TapLockTurbo:OnEvent(e)
	if e == 'SHIP_SPAWNED' then
		--ship just entered sector, so clear stuff out
		self.pressed = false
		self.locked = false
		self:disengage_turbo()
	end
end


--display help
function TapLockTurbo:help()
	print(command_color..'/taplockturbo help'..reset)
	print(info_color..'Displays this message'..reset)
	print(command_color..'/taplockturbo <number>'..reset)
	print(info_color..'Sets the maximum delay between taps to <number> milliseconds'..reset)
	print(info_color..'(Current delay: '..reset..command_color..self.delay..reset..info_color..' ms)'..reset)
	print(info_color..'Bind '..reset..command_color..'+taplockturbo'..reset..info_color..' to a key to use'..reset)
end


--process commands
function TapLockTurbo:process(_,data)
	if (data ~= nil and data[1] == 'help') then
		self:help()
	elseif (data ~= nil and data[1] ~= nil) then
		self:set_delay(tonumber(data[1]))
	else
		self:help()
	end
end


--register commands
RegisterUserCommand('taplockturbo', function(_,data) TapLockTurbo:process(_,data) end)
RegisterUserCommand('+taplockturbo', function(_,data) TapLockTurbo:press() end)
RegisterUserCommand('-taplockturbo', function(_,data) TapLockTurbo:release() end)
--register events
RegisterEvent(TapLockTurbo, 'SHIP_SPAWNED')

